﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Optimization;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Problems.VehicleRouting.ProblemInstances;

namespace HeuristicLab.Problems.VehicleRouting.Interfaces {
  public interface IVRPProblemInstance: IItem {
    IValueParameter<IVRPEvaluator> EvaluatorParameter { get; }
    IVRPEvaluator MoveEvaluator { get; }
    IValueParameter<IVRPCreator> SolutionCreatorParameter { get; }
    IValueParameter<DoubleValue> BestKnownQualityParameter { get; }
    IValueParameter<VRPSolution> BestKnownSolutionParameter { get; }
    IEnumerable<IOperator> Operators { get; }

    DoubleMatrix Coordinates { get; }
    DoubleMatrix DistanceMatrix { get; }
    BoolValue UseDistanceMatrix { get; }
    IntValue Vehicles { get; }
    DoubleArray Demand { get; }
    IntValue Cities { get; }

    DoubleValue FleetUsageFactor { get; }
    DoubleValue DistanceFactor { get; }

    double GetDistance(int start, int end);
    bool Feasible(IVRPEncoding solution);
    bool Feasible(Tour tour);
    VRPEvaluation Evaluate(IVRPEncoding solution);
    VRPEvaluation Evaluate(Tour tour);
    bool Feasible(VRPEvaluation eval);
  }
}
